#!/usr/bin/env python
# -------------------
# tab-width=4
# tab-type=tab
# -------------------
# version=0.1.2

import nautilus

import os
import urllib
import gconf


class ColumnExtension(nautilus.ColumnProvider, nautilus.InfoProvider):
	def __init__(self):
		if os.getuid() == 0:
			self.destroy()
		gconf_dir = "/apps/nautilus/preferences/default_folder_viewer"
		gconf_client = gconf.client_get_default()
		gconf_client.notify_add(gconf_dir, self.get_view_mode, gconf_dir)
		self.get_view_mode(gconf_client, None, None, gconf_dir)

	def get_columns(self):
		return nautilus.Column("NautilusPython::total_size_column",
					"total_size",
					"Total size",
					"Get the total size"),

	def update_file_info(self, item):
		# Return nothing, if gconf says list_view isn't active
		if self.view_mode != "list_view":
			return
		# XDG dirs are not returned at first.
		# A workaround is to simply hit the refresh button in nautilus.
		# Bug is reported to gnome bugtracker as of nautlius-python 0.5.1.
		if item.get_uri_scheme() != 'file':
			return
		item_path = urllib.unquote(item.get_uri()[7:])
		item_total_size = self.get_item_size(item_path)
		item.add_string_attribute('total_size', str(item_total_size))

	def get_view_mode(self, gconf_client, gconf_id, gconf_entry, gconf_dir):
		self.view_mode = gconf_client.get_string(gconf_dir)

	def get_item_size(self, item):
		# Dont include items owned by root, other than those under
		# /media, /mnt, with write permissions (fat32/ntfs drives).
		if os.stat(item).st_uid == 0:
			if item.startswith("/media") or item.startswith("/mnt"):
				if not os.access(item, os.W_OK):
					return "-"
			else: 
				return "-"
		# Dont walk items in gvfs/fuse mountpoints.
		if ".gvfs" in item:
			return "-"
		if os.path.isdir(item):
			folder_size = 0
			for (path, dirs, files) in os.walk(item):
				# Prevent os.walk from walking into gvfs/fuse mountpoints.
				if ".gvfs" in dirs:
					index = dirs.index(".gvfs")
					del dirs[index]
				for file in files:
					filename = os.path.join(path, file)
					try:
						folder_size += os.path.getsize(filename)
					except:
						pass
			pretty_size = self.return_size(folder_size)
		else:
			try:
				pretty_size = self.return_size(os.path.getsize(item))
			except:
				pretty_size = "Error"
		return pretty_size

	def return_size(self, size):
			if size < 1024*1024:
				return "%0.1f KB" % (size/1024.0)
			elif size >= 1024*1024 and size <= 1024*1024*1024:
				return "%0.1f MB" % (size/(1024*1024.0))
			elif size > 1024*1024*1024:
				return "%0.1f GB" % (size/(1024*1024*1024.0))
